@@ -14,16 +14,26 @@ use crate::debug::DebugSymbol;
 // these are limited to u16 right now because they are loaded by opcodes
 pub type ChunkID = u16;
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub enum Chunk {
+    Main,
+    ChunkID(ChunkID),
+}
 
-// pub enum ChunkTag {
-//     Module,
-//     Function,
-//     DefaultArg,
-// }
+impl From<ChunkID> for Chunk {
+    fn from(chunk_id: ChunkID) -> Self {
+        Self::ChunkID(chunk_id)
+    }
+}
 
-#[derive(Default, Debug, Clone)]
-pub struct ChunkInfo {
-    pub symbol: Option<DebugSymbol>,
+
+#[derive(Debug, Clone)]
+pub enum ChunkInfo {
+    ModuleMain,
+    Function {
+        id: FunctionID,
+        symbol: Option<DebugSymbol>,
+    },
 }
 
 /// A buffer used by ChunkBuilder
@@ -105,7 +115,7 @@ impl ChunkBuilder {
     
     pub fn with_strings(strings: StringInterner) -> Self {
         Self {
-            main: ChunkBuf::new(ChunkInfo::default()),
+            main: ChunkBuf::new(ChunkInfo::ModuleMain),
             chunks: Vec::new(),
             functions: Vec::new(),
             consts: Vec::new(),
@@ -116,28 +126,26 @@ impl ChunkBuilder {
     
     // Bytecode
     
-    pub fn new_chunk(&mut self, info: ChunkInfo) -> CompileResult<ChunkID> {
+    pub fn new_chunk(&mut self, info: ChunkInfo) -> CompileResult<Chunk> {
         let chunk_id = ChunkID::try_from(self.chunks.len())
             .map_err(|_| CompileError::from(ErrorKind::ChunkCountLimit))?;
         
         self.chunks.push(ChunkBuf::new(info));
         
-        Ok(chunk_id)
+        Ok(chunk_id.into())
     }
 
-    pub fn chunk(&self, chunk_id: Option<ChunkID>) -> &ChunkBuf { 
-        if let Some(chunk_id) = chunk_id {
-            &self.chunks[usize::from(chunk_id)]
-        } else {
-            &self.main
+    pub fn chunk(&self, chunk_id: Chunk) -> &ChunkBuf {
+        match chunk_id {
+            Chunk::Main => &self.main,
+            Chunk::ChunkID(id) => &self.chunks[usize::from(id)],
         }
     }
     
-    pub fn chunk_mut(&mut self, chunk_id: Option<ChunkID>) -> &mut ChunkBuf { 
-        if let Some(chunk_id) = chunk_id {
-            &mut self.chunks[usize::from(chunk_id)]
-        } else {
-            &mut self.main
+    pub fn chunk_mut(&mut self, chunk_id: Chunk) -> &mut ChunkBuf { 
+        match chunk_id {
+            Chunk::Main => &mut self.main,
+            Chunk::ChunkID(id) => &mut self.chunks[usize::from(id)],
         }
     }
     
@@ -221,7 +229,7 @@ impl ChunkBuilder {
 // TODO store all chunk bytes in a single array
 // TODO figure out how debug symbols will work, esp. at runtime
 
-#[derive(Debug, Default, Clone)]
+#[derive(Debug, Clone)]
 pub struct ChunkIndex {
     info: ChunkInfo,
     offset: usize,
@@ -323,6 +331,11 @@ impl ProgramData {
         &self.chunks[index.as_range()]
     }
     
+    pub fn chunk_info(&self, chunk_id: ChunkID) -> &ChunkInfo {
+        let index = &self.chunk_index[usize::from(chunk_id)];
+        &index.info
+    }
+    
     pub fn get_const(&self, index: ConstID) -> &Constant {
         &self.consts[usize::from(index)]
     }
