@@ -2,7 +2,7 @@
 
 use std::mem;
 use std::any::Any;
-use std::ptr::NonNull;
+use std::ptr::{self, NonNull};
 use std::cell::{Cell, RefCell};
 use log;
 
@@ -22,19 +22,25 @@ struct GCBox<T> where T: GCTrace + ?Sized + 'static {
 
 impl<T> GCBox<T> where T: GCTrace + ?Sized {
     fn value(&self) -> &T { &self.data }
-}
-
-impl<T> GCBox<T> where T: GCTrace + ?Sized {
+    
     #[inline]
     fn size(&self) -> usize {
-        mem::size_of_val(self) + self.value().extra_size()
+        mem::size_of_val(self) + self.value().size_hint()
+    }
+    
+    fn ptr_eq(&self, other: &GCBox<T>) -> bool {
+        // in case T is a trait object, work around for https://github.com/rust-lang/rust/issues/46139
+        ptr::eq(
+            ptr::addr_of!(self.marked),
+            ptr::addr_of!(other.marked),
+        )
     }
 }
 
 pub trait GCTrace {
     /// if the GC'd data owns any allocations, this should return the extra allocated size
     #[inline]
-    fn extra_size(&self) -> usize { 0 }
+    fn size_hint(&self) -> usize { 0 }
 }
 
 
