@@ -6,18 +6,19 @@
 ///! Importing a Sphinx module simply means executing a Sphinx sub-program and binding the
 ///! resulting module to a name.
 
+use std::fmt;
+use std::path::PathBuf;
 use std::cell::{RefCell, Ref, RefMut};
 use std::hash::{Hash, Hasher, BuildHasher};
-use std::collections::HashMap;
 use crate::source::ModuleSource;
 use crate::language::FloatType;
-use crate::runtime::{Variant, DefaultBuildHasher};
+use crate::runtime::{Variant, HashMap, DefaultBuildHasher};
 use crate::runtime::gc::{GC, GCTrace};
 use crate::runtime::types::function::Function;
 use crate::runtime::strings::StringSymbol;
 use crate::runtime::errors::{ExecResult, RuntimeError, ErrorKind};
 
-pub use crate::codegen::{ProgramData, Constant, ConstID, ChunkID};
+pub use crate::codegen::{ProgramData, Constant, ConstID, Chunk, ChunkID};
 
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -34,7 +35,7 @@ pub struct Variable {
 
 #[derive(Debug, Clone)]
 pub struct Namespace {
-    store: HashMap<StringSymbol, Variable, DefaultBuildHasher>,
+    store: HashMap<StringSymbol, Variable>,
 }
 
 impl Default for Namespace {
@@ -97,6 +98,10 @@ impl From<Namespace> for GlobalEnv {
 }
 
 impl GlobalEnv {
+    pub fn new() -> Self {
+        Self::default()
+    }
+    
     pub fn allocate() -> GC<Self> {
         GC::allocate(Self::default())
     }
@@ -113,38 +118,38 @@ impl GlobalEnv {
 impl GCTrace for GlobalEnv { }
 
 
-
 #[derive(Debug)]
 pub struct Module {
-    name: String,
+    ident: ModuleIdent,
     source: Option<ModuleSource>,
     data: ProgramData,
     globals: GC<GlobalEnv>,
 }
 
-impl GCTrace for Module {
-    fn extra_size(&self) -> usize {
-        std::mem::size_of_val(self.name.as_str())
-    }
-}
+impl GCTrace for Module { }
 
 impl Module {
     pub fn allocate(source: Option<ModuleSource>, data: ProgramData) -> GC<Self> {
         let globals = GlobalEnv::allocate();
-        Self::with_globals(globals, source, data)
+        Self::with_env(source, data, globals)
     }
     
-    pub fn with_globals(globals: GC<GlobalEnv>, source: Option<ModuleSource>, data: ProgramData) -> GC<Self> {
+    pub fn with_env(source: Option<ModuleSource>, data: ProgramData, globals: GC<GlobalEnv>) -> GC<Self> {
+        let ident = 
+            if let Some(source) = source.as_ref() { ModuleIdent::from(source) }
+            else { ModuleIdent::from(globals) };
+        
         let module = Self {
-            name: Self::get_name(source.as_ref()),
+            ident,
             source,
             data,
             globals,
         };
+        
         GC::allocate(module)
     }
     
-    pub fn name(&self) -> &str { self.name.as_str() }
+    pub fn ident(&self) -> &ModuleIdent { &self.ident }
     
     pub fn source(&self) -> Option<&ModuleSource> { self.source.as_ref() }
     
@@ -170,13 +175,62 @@ impl Module {
             }
         }
     }
-    
-    fn get_name(source: Option<&ModuleSource>) -> String {
-        if let Some(ModuleSource::File(path)) = source {
-            path.display().to_string()
-        }
-        else {
-            "<anonymous module>".to_string()
+}
+
+
+/// Used to uniquely identify a module
+#[derive(Debug, PartialEq, Eq, Hash)]
+pub enum ModuleIdent {
+    SourcePath(PathBuf), // canonicalized path to the source file
+    SourceHash(u64),  // hash of source text
+    RefHash(u64),  // GC address of globals, for Native Modules
+}
+
+lazy_static! {
+    static ref IDENT_HASH: DefaultBuildHasher = DefaultBuildHasher::default();
+}
+
+impl From<&ModuleSource> for ModuleIdent {
+    fn from(source: &ModuleSource) -> Self {
+        match source {
+            ModuleSource::File(path) => {
+                Self::SourcePath(path.canonicalize().expect("invalid source path"))
+            },
+            
+            ModuleSource::String(text) => {
+                let mut state = IDENT_HASH.build_hasher();
+                text.hash(&mut state);
+                let hash = state.finish();
+                Self::SourceHash(hash)
+            }
         }
     }
 }
+
+impl From<GC<GlobalEnv>> for ModuleIdent {
+    fn from(env: GC<GlobalEnv>) -> Self {
+        let mut state = IDENT_HASH.build_hasher();
+        <GC<GlobalEnv> as Hash>::hash(&env, &mut state);
+        let hash = state.finish();
+        Self::RefHash(hash)
+    }
+}
+
+impl fmt::Display for ModuleIdent {
+    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::SourcePath(path) => {
+                let prefix = std::env::current_dir().ok();
+                
+                let path = prefix.map(|prefix| path.strip_prefix(prefix).ok()).flatten()
+                    .unwrap_or(path);
+                
+                write!(fmt, "\"{}\"", path.display())
+            },
+            
+            Self::SourceHash(hash) => write!(fmt, "#{:016X}", hash),
+            
+            Self::RefHash(hash) => write!(fmt, "&{:016X}", hash),
+        }
+    }
+}
\ No newline at end of file
