@@ -1,7 +1,8 @@
 use std::fs;
 use std::path::{PathBuf, Path};
 use std::io;
-use crate::utils::ReadChars;
+use std::fmt;
+use crate::utils::{self, ReadChars};
 
 use crate::lexer::LexerBuilder;
 use crate::parser::{Parser, ParserError};
@@ -16,16 +17,6 @@ pub enum ModuleSource {
     File(PathBuf),
 }
 
-#[derive(Debug)]
-pub enum SourceText {
-    String(String),
-    File(ReadFileChars),
-}
-
-impl<S> From<S> for SourceText where S: ToString {
-    fn from(text: S) -> Self { SourceText::String(text.to_string()) }
-}
-
 impl ModuleSource {
     // Load the source text
     pub fn read_text(&self) -> io::Result<SourceText> {
@@ -40,10 +31,21 @@ impl ModuleSource {
         let reader = io::BufReader::new(file);
         Ok(ReadChars::new(reader))
     }
-    
 }
 
 
+#[derive(Debug)]
+pub enum SourceText {
+    String(String),
+    File(ReadFileChars),
+}
+
+impl<S> From<S> for SourceText where S: ToString {
+    fn from(text: S) -> Self { SourceText::String(text.to_string()) }
+}
+
+
+
 /// High-level Parsing Interface
 ///
 /// Contains the state required for parsing, and deals with the separate code paths taken for different SourceTypes
